/* Part3 : About Ring Topology 
 * Consider a set of processes arranged in a ring.
 * Each processor stores its rank in MPI_COMM_WORLD in an integer and
 * sends this value onto the processor on its right.
 * The processors continue passing on the values they receive 
 * until they get their own rank back. Each process should finish
 * by printing out the sum of the values.
 *
 */

/* The Definition of MPI */
#include "mpi.h"

void main( int argc, char **argv )
{
	int rank_value;     /* send and return value to/from the process */
	int total_rank = 0;		/* total rank */
	int predecessor, successor, tag = 0;
	int my_rank, num_processes;
	double start, finish, time;

	MPI_Request send_request;  /* Request for SEND */
	MPI_Request recv_request;  /* Request for RECEIVE */

	MPI_Status status;

	/* start up MPI	*/
	MPI_Init( &argc, &argv );

	/* get process rank */
	MPI_Comm_rank( MPI_COMM_WORLD, &my_rank );

	/* get number of processes */
	MPI_Comm_size( MPI_COMM_WORLD, &num_processes );

	successor = ( my_rank + 1 ) % num_processes;
	predecessor = ( my_rank - 1 + num_processes ) % num_processes;
	rank_value = my_rank;

	/* set up the starting time */
	start = MPI_Wtime();

	/* Each process sends its rank to the next processor until they get their own rank back*/
	do {
		MPI_Isend( &rank_value, 1, MPI_INT, successor, tag, MPI_COMM_WORLD, &send_request );
		MPI_Irecv( &rank_value, 1, MPI_INT, predecessor, tag, MPI_COMM_WORLD, &recv_request );

		MPI_Wait(&send_request, &status);
		MPI_Wait(&recv_request, &status);

		total_rank += rank_value;

	} while ( rank_value != my_rank );

	/* stop the time */
	finish = MPI_Wtime();
        
        time = start - finish;

        printf("Rank\tTotal Time\tTotal Value\n");

	printf( "%d\t%f\t%d\n", my_rank, time, total_rank );

	/* close MPI*/
	MPI_Finalize();
}
